@@ -6,9 +6,10 @@
  * later) as published by the Free Software Foundation.
  */
 
+#include <nft.h>
+
 #include <stdio.h>
 #include <unistd.h>
-#include <stdlib.h>
 #include <sys/time.h>
 #include <time.h>
 #include <inttypes.h>
